""" Handler for running selenium and chrome backend """
import platform
from time import sleep
import http
import urllib
import urllib3
import docker
from selenium import webdriver

ARM_IMAGE = "seleniarm/standalone-chromium:105.0"
X86_IMAGE = "selenium/standalone-chrome:105.0"


class DockerHandler:
    """Docker handler"""

    def __init__(self):
        self.client = docker.from_env()
        self.arch = platform.machine()
        self.image = self.select_image(self.arch)
        self.container = None

    @staticmethod
    def select_image(arch):
        """Select the correct image based on the client architecture"""
        if arch == "arm64":
            return ARM_IMAGE
        return X86_IMAGE

    def pull_image(self):
        """Pull the selenium image"""
        self.client.images.pull(self.image)

    def run_container(self):
        """Run the selenium image in a container"""
        self.container = self.client.containers.run(
            self.image,
            auto_remove=True,
            detach=True,
            shm_size="2g",
            ports={"4444/tcp": 4444},
        )

        # Ensure selenium api is responsive
        for i in range(5):
            try:
                with urllib.request.urlopen("http://localhost:4444/") as page:
                    if page.getcode() == 200:
                        break
            except (http.client.RemoteDisconnected, urllib.error.URLError):
                pass
            if i == 4:
                raise RuntimeError(
                    "Selenium Docker API did not return a 200 code within 5 seconds"
                )
            sleep(1)

        # Ensure chrome driver is responsive
        options = webdriver.ChromeOptions()
        options.add_argument("--ignore-ssl-errors=yes")
        options.add_argument("--ignore-certificate-errors")
        for i in range(10):
            try:
                with webdriver.Remote(
                    command_executor="http://localhost:4444/wd/hub", options=options
                ) as _:
                    break
            except urllib3.exceptions.ProtocolError:
                pass
            if i == 4:
                raise RuntimeError("Chrome Docker API did not respond within 5 seconds")
            sleep(1)

    def rm_container(self):
        """Remove the container"""
        self.container.remove(force=True)



""" Module containing methods to process `date` related """
import re
from datetime import datetime, timedelta
import dateutil.parser


def get_working_dates(start, count, cal):
    """get workdays from `start` date to `start+count` date"""
    working_dates = []
    for i in range(count):
        current_date = start + timedelta(days=i)
        current_date = current_date.date()
        year = current_date.year
        holidays = cal.holidays(year)
        holidates = [date for (date, _) in holidays]
        if current_date.isoweekday() < 6:
            if current_date not in holidates:
                working_dates.append((current_date, 8))  # 8: number of working hours
            else:
                working_dates.append((current_date, 0))  # 0: number of working hours
    return working_dates


def get_start_date(start_date_arg):
    """parse user's `start` argument"""
    if start_date_arg.lower() == "today":
        start_date = datetime.today()
    else:
        # PR#22 Parsing Dates with dateutil
        numbers = re.split("[-/ ]", start_date_arg)
        # 4 Digit Year
        if max(map(len, numbers)) == 4:
            yearfirst = len(numbers[0]) == 4
            dayfirst = not yearfirst
            start_date = dateutil.parser.parse(
                start_date_arg, yearfirst=yearfirst, dayfirst=dayfirst
            )
        # 2 Digit Year
        else:
            cand1 = dateutil.parser.parse(start_date_arg, dayfirst=True)  # DMY
            cand2 = dateutil.parser.parse(start_date_arg, yearfirst=True)  # YMD
            today = datetime.today()
            diff1 = abs((cand1 - today).total_seconds())
            diff2 = abs((cand2 - today).total_seconds())
            # Select Nearest
            if diff1 < diff2:
                start_date = cand1
            else:
                start_date = cand2
    return start_date



""" Entry point for cli """
import logging
import os
import sys
import argparse
import warnings
from workalendar.asia import Singapore
from tp_timesheet.docker_handler import DockerHandler
from tp_timesheet.submit_form import submit_timesheet
from tp_timesheet.date_utils import get_working_dates, get_start_date
from tp_timesheet.schedule import ScheduleForm
from tp_timesheet.config import Config

logger = logging.getLogger(__name__)


def parse_args():
    """Parse arguments from the command line"""
    parser = argparse.ArgumentParser(
        description="TP Automated Timesheet Submission - CLI Tool"
    )
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "-s",
        "--start",
        type=str,
        help="Normal mode: Indicating first date to submit a timesheet. \
                Accepted arguments = ['Day/Month/Year', 'Day/Month/Year', today]. \
                NOTE: 'Month/Day/Year' is not accepted!",
    )
    group.add_argument(
        "-a",
        "--automate",
        type=str,
        help="Automate mode: Schedules the form submission to run automatically. Accepted arguments = [weekdays]",
    )
    parser.add_argument(
        "-c",
        "--count",
        type=int,
        required=False,
        default=1,
        help="Number of weekdays to submit a timesheet for, use '5' on a monday to submit for the entire week",
    )
    parser.add_argument(
        "-n",
        "--notification",
        action="store_true",
        help="Notification feature, notification will be shown when the timesheet submission is done(for OSX only)",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="Verbose mode, prints logs and saves screenshots of the timesheet submission page to your desktop",
    )
    parser.add_argument(
        "-d",
        "--dry-run",
        action="store_true",
        help="Dry run mode, runs through as per normal but will not click submit",
    )
    return parser.parse_args()


def run():
    """Entry point"""
    args = parse_args()

    config = Config(verbose=args.verbose)
    # Automate Mode
    if args.automate is not None:
        valid_options = ["weekdays"]
        if args.automate not in valid_options:
            logger.error(
                "'%s' is not a valid option for --automate mode", args.automate
            )
            return
        scheduler = ScheduleForm()
        scheduler.schedule()
        return

    # Normal Mode
    if not args.verbose:
        warnings.filterwarnings(
            "ignore", message="Please take note that, due to arbitrary decisions, "
        )
    cal = Singapore()

    start_date = get_start_date(args.start)
    dates = get_working_dates(start=start_date, count=args.count, cal=cal)

    if not dates:
        logger.info(
            "Submitted dates fall on weekends, form submission is not required."
        )
        return

    string_list = [f"{date}: {hours} hours" for (date, hours) in dates]
    logger.info(
        "Date(s) (yyyy-mm-dd) to be submitted for %s: %s", config.EMAIL, string_list
    )

    docker_handler = DockerHandler()
    try:
        logger.debug("Pulling latest image")
        docker_handler.pull_image()
        logger.debug("Launching docker container for selenium backend")
        docker_handler.run_container()

        for (date, hours) in dates:
            submit_timesheet(
                config.URL,
                config.EMAIL,
                date,
                verbose=args.verbose,
                dry_run=args.dry_run,
                working_hours=hours,
            )

        # Notification (OSX only)
        if args.notification and sys.platform.lower() == "darwin":
            if len(dates) == 1:
                notification_text = (
                    f"Timesheet for {args.start.lower()} is successfully submitted."
                )
            else:
                notification_text = f"Timesheets from {dates[0]} to {dates[-1]} are successfully submitted."
            if args.dry_run:
                notification_text = f"[DRY_RUN] {notification_text}"
            os.system(
                f"""osascript -e 'display notification "{notification_text}" with title "TP Timesheet"'"""
            )

    finally:
        logger.debug("Cleaning up docker container")
        if docker_handler.container is not None:
            docker_handler.rm_container()


if __name__ == "__main__":
    run()
